@@ -3,55 +3,43 @@
 
 # Memoize.pm
 #
-# Transparent memoization of idempotent functions
-#
 # Copyright 1998, 1999, 2000, 2001, 2012 M. J. Dominus.
 # You may copy and distribute this program under the
 # same terms as Perl itself.
 
+use strict; use warnings;
+
 package Memoize;
-$VERSION = '1.08';
+our $VERSION = '1.09';
 
 # Compile-time constants
 sub SCALAR () { 0 } 
 sub LIST () { 1 } 
 
-
-#
-# Usage memoize(functionname/ref,
-#               { NORMALIZER => coderef, INSTALL => name,
-#                 LIST_CACHE => descriptor, SCALAR_CACHE => descriptor }
-#
-
 use Carp;
 use Exporter;
-use vars qw($DEBUG);
+our $DEBUG;
 use Config;                     # Dammit.
-@ISA = qw(Exporter);
-@EXPORT = qw(memoize);
-@EXPORT_OK = qw(unmemoize flush_cache);
-use strict;
+*import = \&Exporter::import;
+our @EXPORT = qw(memoize);
+our @EXPORT_OK = qw(unmemoize flush_cache);
 
 my %memotable;
-my %revmemotable;
 my @CONTEXT_TAGS = qw(MERGE TIE MEMORY FAULT HASH);
 my %IS_CACHE_TAG = map {($_ => 1)} @CONTEXT_TAGS;
 
 sub CLONE {
   my @info = values %memotable;
-  %revmemotable = map +($_->{MEMOIZED} => ''.$_->{U}), @info;
-  %memotable = map +($_->{U} => $_), @info;
+  %memotable = map +($_->{WRAPPER} => $_), @info;
 }
 
 # Raise an error if the user tries to specify one of thesepackage as a
 # tie for LIST_CACHE
-
 my %scalar_only = map {($_ => 1)} qw(DB_File GDBM_File SDBM_File ODBM_File NDBM_File);
 
 sub memoize {
   my $fn = shift;
   my %options = @_;
-  my $options = \%options;
 
   unless (defined($fn) && 
 	  (ref $fn eq 'CODE' || ref $fn eq '')) {
@@ -59,53 +47,39 @@ sub memoize {
   }
 
   my $uppack = caller;		# TCL me Elmo!
-  my $cref;			# Code reference to original function
   my $name = (ref $fn ? undef : $fn);
-
-  # Convert function names to code references
-  $cref = &_make_cref($fn, $uppack);
-
-  # Locate function prototype, if any
+  my $cref = _make_cref($fn, $uppack);
   my $proto = prototype $cref;
-  if (defined $proto) { $proto = "($proto)" }
-  else { $proto = "" }
+  $proto = defined $proto ? "($proto)" : '';
 
   # I would like to get rid of the eval, but there seems not to be any
   # other way to set the prototype properly.  The switch here for
   # 'usethreads' works around a bug in threadperl having to do with
   # magic goto.  It would be better to fix the bug and use the magic
   # goto version everywhere.
+  my $info;
   my $wrapper = 
       $Config{usethreads} 
-        ? eval "sub $proto { &_memoizer(\$cref, \@_); }" 
-        : eval "sub $proto { unshift \@_, \$cref; goto &_memoizer; }";
+        ? eval "sub $proto { &_memoizer(\$info, \@_); }"
+        : eval "sub $proto { unshift \@_, \$info; goto &_memoizer; }";
 
   my $normalizer = $options{NORMALIZER};
   if (defined $normalizer  && ! ref $normalizer) {
     $normalizer = _make_cref($normalizer, $uppack);
   }
 
-  my $install_name;
-  if (defined $options->{INSTALL}) {
-    # INSTALL => name
-    $install_name = $options->{INSTALL};
-  } elsif (! exists $options->{INSTALL}) {
-    # No INSTALL option provided; use original name if possible
-    $install_name = $name;
-  } else {
-    # INSTALL => undef  means don't install
-  }
+  my $install_name = exists $options{INSTALL}
+    ? $options{INSTALL} # use given name (or, if undef: do not install)
+    : $name; # no INSTALL option provided: default to original name if possible
 
   if (defined $install_name) {
     $install_name = $uppack . '::' . $install_name
 	unless $install_name =~ /::/;
     no strict;
-    local($^W) = 0;	       # ``Subroutine $install_name redefined at ...''
+    no warnings 'redefine';
     *{$install_name} = $wrapper; # Install memoized version
   }
 
-  $revmemotable{$wrapper} = "" . $cref; # Turn code ref into hash key
-
   # These will be the caches
   my %caches;
   for my $context (qw(SCALAR LIST)) {
@@ -152,23 +126,23 @@ sub memoize {
     my $context;
     foreach $context (qw(SCALAR LIST)) {
       # If the relevant option wasn't `TIE', this call does nothing.
-      _my_tie($context, $caches{$context}, $options);  # Croaks on failure
+      _my_tie($context, $caches{$context}, $options{"${context}_CACHE"}); # Croaks on failure
     }
   }
 
-  # We should put some more stuff in here eventually.
-  # We've been saying that for several versions now.
-  # And you know what?  More stuff keeps going in!
-  $memotable{$cref} = 
+  $info =
   {
-    O => $options,  # Short keys here for things we need to access frequently
     N => $normalizer,
     U => $cref,
-    MEMOIZED => $wrapper,
-    PACKAGE => $uppack,
     NAME => $install_name,
     S => $caches{SCALAR},
     L => $caches{LIST},
+    MERGED => $options{MERGED},
+  };
+
+  $memotable{$wrapper} = {
+    INFO    => $info,
+    WRAPPER => $wrapper, # cannot be in $info because $wrapper captures $info
   };
 
   $wrapper			# Return just memoized version
@@ -176,18 +150,15 @@ sub memoize {
 
 # This function tries to load a tied hash class and tie the hash to it.
 sub _my_tie {
-  my ($context, $hash, $options) = @_;
-  my $fullopt = $options->{"${context}_CACHE"};
+  my ($context, $hash, $fullopt) = @_;
 
   # We already checked to make sure that this works.
-  my $shortopt = (ref $fullopt) ? $fullopt->[0] : $fullopt;
+  my ($shortopt, @args) = ref $fullopt ? @$fullopt : $fullopt;
 
   return unless defined $shortopt && $shortopt eq 'TIE';
   carp("TIE option to memoize() is deprecated; use HASH instead")
-      if $^W;
+      if warnings::enabled('all');
 
-  my @args = ref $fullopt ? @$fullopt : ();
-  shift @args;
   my $module = shift @args;
   if ($context eq 'LIST' && $scalar_only{$module}) {
     croak("You can't use $module for LIST_CACHE because it can only store scalars");
@@ -204,7 +175,7 @@ sub _my_tie {
 
 sub flush_cache {
   my $func = _make_cref($_[0], scalar caller);
-  my $info = $memotable{$revmemotable{$func}};
+  my $info = $memotable{$func}{INFO};
   die "$func not memoized" unless defined $info;
   for my $context (qw(S L)) {
     my $cache = $info->{$context};
@@ -221,8 +192,7 @@ sub flush_cache {
 
 # This is the function that manages the memo tables.
 sub _memoizer {
-  my $orig = shift;		# stringized version of ref to original func.
-  my $info = $memotable{$orig};
+  my $info = shift;
   my $normalizer = $info->{N};
 
   my $argstr;
@@ -238,7 +208,7 @@ sub _memoizer {
       croak "Internal error \#41; context was neither LIST nor SCALAR\n";
     }
   } else {                      # Default normalizer
-    local $^W = 0;
+    no warnings 'uninitialized';
     $argstr = join chr(28),@_;  
   }
 
@@ -246,12 +216,12 @@ sub _memoizer {
     my $cache = $info->{S};
     _crap_out($info->{NAME}, 'scalar') unless $cache;
     if (exists $cache->{$argstr}) { 
-      return $info->{O}{MERGED}
+      return $info->{MERGED}
         ? $cache->{$argstr}[0] : $cache->{$argstr};
     } else {
       my $val = &{$info->{U}}(@_);
       # Scalars are considered to be lists; store appropriately
-      if ($info->{O}{MERGED}) {
+      if ($info->{MERGED}) {
 	$cache->{$argstr} = [$val];
       } else {
 	$cache->{$argstr} = $val;
@@ -278,35 +248,21 @@ sub unmemoize {
   my $uppack = caller;
   my $cref = _make_cref($f, $uppack);
 
-  unless (exists $revmemotable{$cref}) {
+  unless (exists $memotable{$cref}) {
     croak "Could not unmemoize function `$f', because it was not memoized to begin with";
   }
 
-  my $tabent = $memotable{$revmemotable{$cref}};
+  my $tabent = $memotable{$cref}{INFO};
   unless (defined $tabent) {
     croak "Could not figure out how to unmemoize function `$f'";
   }
   my $name = $tabent->{NAME};
   if (defined $name) {
     no strict;
-    local($^W) = 0;	       # ``Subroutine $install_name redefined at ...''
+    no warnings 'redefine';
     *{$name} = $tabent->{U}; # Replace with original function
   }
-  delete $memotable{$revmemotable{$cref}};
-  delete $revmemotable{$cref};
-
-  # This removes the last reference to the (possibly tied) memo tables
-  # my ($old_function, $memotabs) = @{$tabent}{'U','S','L'};
-  # undef $tabent; 
-
-#  # Untie the memo tables if they were tied.
-#  my $i;
-#  for $i (0,1) {
-#    if (tied %{$memotabs->[$i]}) {
-#      warn "Untying hash #$i\n";
-#      untie %{$memotabs->[$i]};
-#    }
-#  }
+  delete $memotable{$cref};
 
   $tabent->{U};
 }
